//
// Created by epagris on 2023.04.25..
//

#include "custom_ethertype_connblock.h"
#include "../../eth_interface.h"
#include "../../utils.h"
#include "../../dynmem.h"
#include "../../global_state.h"
#include "etherlib/pckt_assembler.h"

static bool filtCET(const PcktSieveFilterCondition *filtCond, const PcktProps *contProps, const PcktProps *ownProps, EthInterface *intf) {
    EthernetProps *ethProps = (EthernetProps *) contProps;
    //MSG("FILT! %X %X\n", ethProps->length_type, ETHERTYPE_FROM_FILTCOND(filtCond));
    return ethProps->length_type == ETHERTYPE_FROM_FILTCOND(filtCond);
}

cbd cet_new_connblock(struct EthInterface_ *intf, uint16_t etherType, SieveCallBackFn cbFn) {
    ConnBlock cetConnB;
    connb_init_defaults(&cetConnB);

    PcktSieveFilterCondition filtCond;
    packfiltcond_zero(&filtCond);
    ETHERTYPE_TO_FILTCOND(&filtCond, etherType);
    PcktSieveLayerTag tag;
    tag.u = 0;
    cetConnB.sieveLayer = packsieve_new_layer(&intf->sieve.layer0, &filtCond, false, filtCET, cbFn, tag, etherType);
    ASSERT_NULL(cetConnB.sieveLayer);
    cetConnB.sieveLayer->connBReportFn = cet_print_report;

    cetConnB.sieve = &intf->sieve;

    // store connection block to CBDT
    cbd d = cbdt_alloc_new(E.cbdt, &cetConnB);
    if (d == CBDT_ERR) { // on error free everything we have allocated before
        packsieve_remove_layer(cetConnB.sieveLayer);
    }

    return d;
}

int cet_send(cbd d, uint8_t *dest, const uint8_t *data, uint16_t size) {
    return cet_send_arg(d, dest, data, size, 0);
}

int cet_send_arg(cbd d, const uint8_t *dest, const uint8_t *data, uint16_t size, uint32_t arg) {
    ConnBlock connBlock;
    if (!cbdt_get_connection_block(E.cbdt, d, &connBlock)) {
        ERROR("Invalid CBD descriptor: '%d'!\n", d);
        return 0;
    }

    PcktHeaderElement *ethHeader = ALLOC_HEADER_ELEMENT(EthernetProps);
    ethHeader->next = NULL;
    ethHeader->prev = NULL;

    EthernetProps *ethProps = HEADER_FETCH_PROPS(EthernetProps, ethHeader);
    ethProps->length_type = ETHERTYPE_FROM_FILTCOND(&connBlock.sieveLayer->filtCond);
    memcpy(ethProps->destAddr, dest, ETH_HW_ADDR_LEN);
    memcpy(ethProps->sourceAddr, connBlock.sieve->intf->mac, ETH_HW_ADDR_LEN);

    ethProps->hdrInsFn = insert_ethernet_header;
    ethProps->headerSize = ETH_ETHERNET_HEADER_SIZE;

    Pckt cooked;
    cooked.payload = data;
    cooked.payloadSize = size;
    cooked.header = ethHeader;

    // NOT FILLED FIELDS
    cooked.headerSize = 0;
    cooked.time_s = 0;
    cooked.time_ns = 0;

    RawPckt raw;
    pckt_assemble(&raw, &cooked, connBlock.sieve->intf);
    raw.ext.tx.arg = arg; // assign argument
    raw.ext.tx.txTsCb = connBlock.sieveLayer->txTsCb; // assign with TX timestamp callback

    ethinf_transmit(connBlock.sieve->intf, &raw);

    release_resources:
    // free headers
    dynmem_free(ethHeader);

    return size;
}

void cet_print_report(const ConnBlock* connBlock) {
    const PcktSieveLayer * sl = connBlock->sieveLayer;
    INFO("EtherType: 0x%X", ETHERTYPE_FROM_FILTCOND(&sl->filtCond));
}
